local addonName, ns = ...

-- Add-on nameSpace
local TAB = LibStub ("AceAddon-3.0"):NewAddon (addonName, "AceEvent-3.0")
local L = LibStub ("AceLocale-3.0"):GetLocale (addonName)
local Dialog 							-- Placeholder for LibStub("AceConfigDialog-3.0")

-- Constants
local TIER_MAX = 7
local COL_MAX = 3
local SLASH_CASTUSE = {[1] = SLASH_CAST1, [2] = SLASH_CAST2, [3] = SLASH_CAST3, [4] = SLASH_CAST4, [5] = SLASH_USE1, [6] = SLASH_USE2}

-- Frames
local F_Temp = CreateFrame ("frame")	-- frame for onUpdate and Blizzard menu

-- Tables
TAB.talents = { }
-- Variables
local options 							-- Placeholder for the options table
local totalTime = 0 					-- onUpdate time holder

local defaults = {
	profile = {
		enabled = true,
		macroEnabled = true,
		chatOutput = true,
		saveActiveSpells = true,
	},
	char = {
		savedActionBarSpells = { },
	},
	class = {
		blacklistMacro = { },
		blacklistSpell = { },
	},
}

--**********************************************************************************************************************************************************
-------------------------------------------------------------- Initialisation Functions
--**********************************************************************************************************************************************************

function TAB:OnInitialize ()
	TAB.db = LibStub ("AceDB-3.0"):New ("TalentActionBarsDB", defaults, true)
	
	TAB.db.RegisterCallback (TAB, "OnProfileChanged", "OnProfileChanged")
	TAB.db.RegisterCallback (TAB, "OnProfileCopied", "OnProfileChanged")
	TAB.db.RegisterCallback (TAB, "OnProfileReset", "OnProfileChanged")
	
	-- OnDemand option load into memory
	F_Temp.name = addonName	
	local button = CreateFrame ("Button", nil, F_Temp, "OptionsButtonTemplate")
	button:SetPoint ("CENTER")
	button:SetText ("/tab config")
	button:SetScript ("OnClick", function ()
			InterfaceOptionsFrameCancel_OnClick ()
			HideUIPanel (GameMenuFrame)
			TAB:SetupOptions ()
	end)
	
	InterfaceOptions_AddCategory (F_Temp)
		
	SLASH_TAB1 = "/tab";
	SlashCmdList ["TAB"] = function (msg)
		local command, rest = msg:match ("^(%S*)%s*(.-)$")
		if command == "" then
			print (" /tab fix|cffaaaaaa - " .. L["Attempt to fix talents in macros and action bars"])			
			print (" /tab config|cffaaaaaa - " .. L["Open configuration"])			
			print (" /tab listbl|cffaaaaaa - " .. L["List blacklisted macros and spells"])			
			print (" /tab blmacro|cffaaaaaa - " .. L["Add/Remove macro based on name to the blacklist"])			
			print (" /tab blspell|cffaaaaaa - " .. L["Add/Remove spell based on name to the blacklist"])	
			print (" /tab blclear|cffaaaaaa - " .. L["Clear all blacklist data for your class"])	
			print (" /tab clearsavedbuttons|cffaaaaaa - " .. L["Clear buttons that are saved when switching between active and passive talents"])			
		end
		if command == "clearsavedbuttons" then
			print ("|cff20ff20Talent Action Bars|r: "..L["Cleared saved buttons"])
			TAB.db.char.savedActionBarSpells = { }
		elseif command == "fix" then
			TAB:fix ()
		elseif command == "enable" then
			TAB:Enable ()
		elseif command == "disable" then
			TAB:Disable ()
		elseif (command == "config") then
			TAB:SetupOptions ()
		elseif (command == "blmacro") then
			TAB:BlackList (rest, TAB.db.class.blacklistMacro)
		elseif (command == "blspell") then
			TAB:BlackList (rest, TAB.db.class.blacklistSpell)
		elseif (command == "listbl") then
			print ("|cff20ff20Talent Action Bars|r: "..L["Blaclisted Macros"])
			for k, v in pairs (TAB.db.class.blacklistMacro) do
				print ("|cffaaaaaa"..k)
			end
			print ("|cff20ff20Talent Action Bars|r: "..L["Blaclisted Spells"])
			for k, v in pairs (TAB.db.class.blacklistSpell) do
				print ("|cffaaaaaa"..k)
			end
		elseif (command == "blclear") then
			TAB.db.class.blacklistMacro = { }
			TAB.db.class.blacklistSpell = { }
			print ("|cff20ff20Talent Action Bars|r: "..L["Blacklist Cleared"])
		end			
	end
end

function TAB:OnEnable ()
	-- TAB:RegisterEvent ("LEARNED_SPELL_IN_TAB")
	TAB:RegisterEvent ("ACTIVE_TALENT_GROUP_CHANGED")
	TAB:RegisterEvent ("PLAYER_TALENT_UPDATE")
	
	local active = GetActiveSpecGroup ()
	for tier = 1, TIER_MAX do
		for col = 1, COL_MAX do	
			local _, name = GetTalentInfo (tier, col, active)	
			TAB.talents [name] = tier
			TAB.talents [tier] = TAB.talents [tier] or { }
			TAB.talents [tier][col] = name
		end
	end
	
	TAB:fix ()
	print ("|cff20ff20Talent Action Bars|r: "..L["Enabled"])
	
	-- Localization notice
	local loadedL = _G.GetLocale()
	if loadedL == "ptBR" or loadedL == "frFR" or loadedL == "deDE" or loadedL == "itIT" or loadedL == "koKR" or loadedL == "esMX" or loadedL == "ruRU" or loadedL == "zhCN" or loadedL == "esES" or loadedL == "zhTW" then
        print("|cff20ff20Talent Action Bars|r: Help Translate at the Project Site!")
    end
end

function TAB:OnDisable ()
	-- TAB:UnregisterEvent ("LEARNED_SPELL_IN_TAB")
	TAB:UnregisterEvent ("ACTIVE_TALENT_GROUP_CHANGED")
	TAB:UnregisterEvent ("PLAYER_TALENT_UPDATE")
	print ("|cff20ff20Talent Action Bars|r: "..L["Disabled"])
end

function TAB:OnProfileChanged()
end

--**********************************************************************************************************************************************************
-------------------------------------------------------------- Body
--**********************************************************************************************************************************************************

function TAB:print (st)
	if TAB.db.profile.chatOutput then print (st) end
end

function GetSpellId (nr)
	local link = GetSpellLink (nr)
	if link == nil then return nil end
	return tonumber (link:match ("spell:(%d+)"))
end

function TAB:BlackList (item, list) 
	if list [item] then
		print (string.format ("|cff20ff20Talent Action Bars|r: "..L["Removed %s from blacklist"], item))
		list [item] = nil
	else
		print (string.format ("|cff20ff20Talent Action Bars|r: "..L["Blacklisted %s"], item))
		list [item] = item
	end
end

function TAB:fix() 
	for tier = 1, TIER_MAX do
		local unspent, id = GetTalentRowSelectionInfo (tier)
		if not unspent then
			local _, name = GetTalentInfoByID (id)
			TAB:talentChanged (GetSpellId (name))
		end
	end
end

function TAB:saveActiveSpells (tier) 
	local specID = GetSpecialization ()
	local active = GetActiveSpecGroup ()
	
	for i = 1,120 do
		local actionType, actionID = GetActionInfo (i) 
		if actionType == "spell" then
			local spellName = GetSpellInfo (actionID)
			if TAB.talents [spellName] and TAB.talents [spellName] == tier then
				TAB.db.char.savedActionBarSpells [specID] = TAB.db.char.savedActionBarSpells [specID] or {}
				TAB.db.char.savedActionBarSpells [specID][tier] = i
			end	
		end
	end		
end

function TAB:talentChanged (learnId) 
	local learnName = GetSpellInfo (learnId)	
	local tier = TAB.talents [learnName]
	if not tier then return end
	
	if IsPassiveSpell (learnName) then 
		if TAB.db.profile.saveActiveSpells then TAB:saveActiveSpells (tier) end		
		return -- Exit because you can't put passive spells on bars
	end
	local specID = GetSpecialization ()
	if TAB.db.profile.saveActiveSpells and TAB.db.char.savedActionBarSpells [specID] then
		if TAB.db.char.savedActionBarSpells [specID][tier] then
			local _, id = GetActionInfo (TAB.db.char.savedActionBarSpells [specID][tier])
			if not id then
				PickupSpell (learnId)
				PlaceAction (TAB.db.char.savedActionBarSpells [specID][tier])
			else
				TAB.db.char.savedActionBarSpells [specID][tier] = nil
			end
		end
	end		
	for i = 1,120 do
		local tpe, id = GetActionInfo (i)
		if tpe == "macro" and TAB.db.profile.macroEnabled then
			local name, texture, body = GetMacroInfo (id)
			if not TAB.db.class.blacklistMacro [name] and body then
				for col = 1, COL_MAX do
					if TAB.talents [tier][col] ~= learnName then
						for i,v in pairs (SLASH_CASTUSE) do
							if string.find (body, v.."(.*)"..TAB.talents [tier][col]) then
								body = string.gsub (body, TAB.talents [tier][col], learnName)
								EditMacro (id, nil, nil, body)
								TAB:print (string.format ("|cff20ff20Talent Action Bars|r: "..L["Replaced %s in macro %s with %s"], TAB.talents [tier][col], name, learnName))
								break
							end
						end
					end
				end
			end
		end
		if tpe == "spell" then
			local name = GetSpellInfo (id)
			if not TAB.db.class.blacklistSpell [name] then
				for col = 1, COL_MAX do
					if name == TAB.talents [tier][col] and TAB.talents [tier][col] ~= learnName then 
						PickupAction (i)
						ClearCursor()
						PickupSpell (learnId)
						PlaceAction (i)
						TAB:print (string.format ("|cff20ff20Talent Action Bars|r: "..L["Replaced %s with %s on action bar spot %d"], name, learnName, i))
					end
				end
			end
		end
	end
	ClearCursor ()
end

local function LF_onUpdate (self, elapsed)
	totalTime = totalTime + elapsed
    if totalTime >= .3 then
		F_Temp:SetScript ("OnUpdate", nil)
		TAB:fix ()
		TAB:RegisterEvent ("PLAYER_TALENT_UPDATE")
    end
end

function TAB:PLAYER_TALENT_UPDATE () -- LEARNED_SPELL_IN_TAB DOES NOT FIRE ON SOME LEARNED SPELLS. BLIZZ PROBLEM. FIX UNTIL RESOLVED
	totalTime = 0
	TAB:UnregisterEvent ("PLAYER_TALENT_UPDATE")
	F_Temp:SetScript ("OnUpdate", LF_onUpdate)
end

function TAB:ACTIVE_TALENT_GROUP_CHANGED ()
	totalTime = 0
	TAB:UnregisterEvent ("PLAYER_TALENT_UPDATE")
	F_Temp:SetScript ("OnUpdate", LF_onUpdate)
end

--function TAB:LEARNED_SPELL_IN_TAB (event, learnId, tab, x1, x2)
--	TAB:talentChanged (learnId)
--end

--**********************************************************************************************************************************************************
-------------------------------------------------------------------- Options Table
--**********************************************************************************************************************************************************

function TAB:SetupOptions ()
	local selectedBlackListMacro, selectedBlackListSpell
	if not options then
		options = {
			name = addonName,
			type = "group",
			name = "Talent Action Bars: "..GetAddOnMetadata (addonName, "Version"),
			get = function (info) return TAB.db.profile [info [#info]] end,
			set = function (info, v) TAB.db.profile [info [#info]] = v end,
			args = {
				general = {
					type = "group",
					name = L["General Settings"],
					order = 0,
					childGroups = "tab",
					args = {
						headerGroup = {
							type = "group",
							name = " ",
							order = 0,
							inline = true,
							args = {
								enabled = {
									type = 'toggle',
									name = L["Enable"],
									desc = L["Enable"].." Talent Action Bars", 
									order = 1,
									set = function (info, v) TAB.db.profile [info [#info]] = v
										if v then TAB:Enable () else TAB:Disable () end
									end,
								},
								fixTalents = {
									type = "execute",
									name = L["Fix"],
									desc = L["Attempt to fix talents in macros and action bars"],
									order = -1,
									func = function () 
										TAB:fix ()
									end,
								},
							},
						},
						macroEnabled = {
							type = 'toggle', width = "full",
							name = L["Replace talent spell names in macros"],
							desc = "",
							order = 2,
						},
						chatOutput = {
							type = 'toggle', width = "full",
							name = L["Report action button and macro changes"],
							desc = "",
							order = 3,
						},
						saveActiveSpells = {
							type = 'toggle', width = "full",
							name = L["Save Active Spells"],
							desc = L["Save active spells on your action bars when switching to passive ones"],
							order = 4,
							set = function (info, v) TAB.db.profile [info [#info]] = v
								TAB:fix()
							end,
						},
						blacklistMacro = {
							type = "group",
							name = L["Blacklist Macros"],
							args = {
								typeBListMacro = {
									type = 'input', width = 'double',
									name = L["Enter Macro"],
									desc = "",
									order = 1,
									get = function (info) return "" end,
									set = function (info, v) TAB:BlackList (v, TAB.db.class.blacklistMacro) end,
								},
								selectBListMacro = {
									type = 'select', width = 'double',
									name = L["Delete Macro"],
									desc = "",
									order = 5,
									values = TAB.db.class.blacklistMacro,
									get = function (info) return selectedBlackListMacro end,
									set = function (info, v) selectedBlackListMacro = v end,
								},
								deleteBListMacro = {
									type = "execute", width = 'half',
									name = L["Delete"],
									desc = "",
									order = 10,
									func = function() 
										TAB:BlackList (selectedBlackListMacro, TAB.db.class.blacklistMacro)
										selectedBlackListMacro = nil
									end,
								},
							},
						},
						blacklistSpell = {
							type = "group",
							name = L["Blacklist Spell"],
							args = {
								enterBListSpell = {
									type = 'input', width = 'double',
									name = L["Enter Spell"],
									desc = "",
									order = 1,
									get = function (info) return "" end,
									set = function (info, v) TAB:BlackList (v, TAB.db.class.blacklistSpell) end,
								},
								selectBListSpell = {
									type = 'select', width = 'double',
									name = L["Delete Spell"],
									desc = "",
									order = 5,
									values = TAB.db.class.blacklistSpell,
									get = function (info) return selectedBlackListSpell end,
									set = function (info, v) selectedBlackListSpell = v end,
								},
								deleteBListSpell = {
									type = "execute", width = 'half',
									name = L["Delete"],
									desc = "",
									order = -1,
									func = function() 
										TAB:BlackList (selectedBlackListSpell, TAB.db.class.blacklistSpell)
										selectedBlackListSpell = nil
									end,
								},
							},
						},
					},
				},
			},
		}
		options.args.profile = LibStub ("AceDBOptions-3.0"):GetOptionsTable (TAB.db)
		
		Dialog = LibStub ("AceConfigDialog-3.0")
		LibStub ("AceConfigRegistry-3.0"):RegisterOptionsTable (addonName, options)
		Dialog:AddToBlizOptions (addonName, options.args.general.name, addonName, "general")
		Dialog:AddToBlizOptions (addonName, options.args.profile.name, addonName, "profile")
		print ("|cff20ff20Talent Action Bars|r: "..L["Option table now loaded into memory"])
	end
	
	if not Dialog.OpenFrames[addonName] then 
		Dialog:SetDefaultSize (addonName, 700, 580)
		Dialog:Open (addonName) 
	else 
		Dialog:Close (addonName) 
	end	
end